@@ -31,6 +31,7 @@
 #include <hip/hip_runtime.h>
 #include <hipblaslt/hipblaslt-ext.hpp>
 #include <hipblaslt/hipblaslt.h>
+#include <hipblaslt_arguments.hpp>
 #include <hipblaslt_vector.hpp>
 #include <iostream>
 #include <limits>
@@ -216,7 +217,7 @@ static void show_usage(char* argv[])
         << "\t-v, --verbose\t\t\t\tVerbose output\n"
         << "\t-V, --validate\t\t\t\tVerify results\n"
         << "\t--initialization \t\t\t\tInitialize matrix data. Options: rand_int, trig_float, "
-           "hpl(floating). (default is hpl)\n"
+           "hpl(floating), special, zero. (default is hpl)\n"
         << "\t--bench_count\t\t\t\tNumber of benchmark runs (default is 1)\n"
         << "\t--sync_count\t\t\t\tNumber of sync runs (default is 1)\n"
         << "\t--in_datatype \t\tdatatype \tGEMM_STRIDED argument in: fp32, fp16, bf16 (default is "
@@ -602,6 +603,13 @@ void initialize_a_b_c_bias(std::vector<TiA>&        ha,
         hipblaslt_init_hpl<Tout>(hc, size_c);
         hipblaslt_init_hpl<float>(h_bias, size_bias);
     }
+    else if(initialization == hipblaslt_initialization::zero)
+    {
+        hipblaslt_init_zero<TiA>(ha, size_a);
+        hipblaslt_init_zero<TiB>(hb, size_b);
+        hipblaslt_init_zero<Tout>(hc, size_c);
+        hipblaslt_init_zero<float>(h_bias, size_bias);
+    }
 }
 
 __global__ void kernelUpdateN(uint32_t gemm_count, void* userArgs, int32_t* sizes_n)
@@ -811,8 +819,8 @@ int test_hipblaslt(hipDataType                 in_datatype,
                   << ", " << ToString(actType[i]) << std::endl;
     }
 
-    std::vector<hipblaslt_ext::GemmEpilogue> gemmEpilogue(gemm_count);
-    std::vector<hipblaslt_ext::GemmInputs>   gemmInputs(gemm_count);
+    std::vector<hipblaslt_ext::GemmEpilogueV2> gemmEpilogue(gemm_count);
+    std::vector<hipblaslt_ext::GemmInputsV2>   gemmInputs(gemm_count);
     for(size_t i = 0; i < gemm_count; i++)
     {
         if(enable_bias[i] && actType[i] == ActivationType::NONE)
@@ -827,24 +835,24 @@ int test_hipblaslt(hipDataType                 in_datatype,
             epilogue[i] = HIPBLASLT_EPILOGUE_RELU;
         else if(!enable_bias[i] && actType[i] == ActivationType::GELU)
             epilogue[i] = HIPBLASLT_EPILOGUE_GELU;
-        gemmEpilogue[i].mode           = epilogue[i];
-        gemmEpilogue[i].bias_data_type = static_cast<hipDataType>(HIP_R_32F);
-        gemmInputs[i].a                = da[i];
-        gemmInputs[i].b                = db[i];
-        gemmInputs[i].c                = dc[i];
-        gemmInputs[i].d                = dd[i];
-        gemmInputs[i].alpha            = static_cast<void*>(&alpha[i]);
-        gemmInputs[i].beta             = static_cast<void*>(&beta[i]);
-        gemmInputs[i].bias             = d_bias[i];
+        gemmEpilogue[i].setMode(epilogue[i]);
+        gemmEpilogue[i].setBiasDataType(static_cast<hipDataType>(HIP_R_32F));
+        gemmInputs[i].setA(da[i]);
+        gemmInputs[i].setB(db[i]);
+        gemmInputs[i].setC(dc[i]);
+        gemmInputs[i].setD(dd[i]);
+        gemmInputs[i].setAlpha(static_cast<void*>(&alpha[i]));
+        gemmInputs[i].setBeta(static_cast<void*>(&beta[i]));
+        gemmInputs[i].setBias(d_bias[i]);
     }
 
-    auto gemmProblemType = hipblaslt_ext::GemmProblemType{trans_a,
-                                                          trans_b,
-                                                          in_datatype,
-                                                          in_datatype,
-                                                          out_datatype,
-                                                          out_datatype,
-                                                          HIPBLAS_COMPUTE_32F};
+    auto gemmProblemType = hipblaslt_ext::GemmProblemTypeV2(trans_a,
+                                                            trans_b,
+                                                            in_datatype,
+                                                            in_datatype,
+                                                            out_datatype,
+                                                            out_datatype,
+                                                            HIPBLAS_COMPUTE_32F);
 
     // step 1: set problem to {Ms, {sum of N, 1, 1, 1, ...}, Ks}
     CHECK_HIPBLASLT_ERROR(groupedGemm.setProblem(m,
@@ -1110,7 +1118,13 @@ int main(int argc, char* argv[])
     m.resize(n.size(), m_0);
     k.resize(n.size(), k_0);
 
-    int32_t gemm_count = m.size();
+    int32_t         gemm_count = m.size();
+    int             deviceId;
+    hipDeviceProp_t deviceProperties;
+    static_cast<void>(hipGetDevice(&deviceId));
+    static_cast<void>(hipGetDeviceProperties(&deviceProperties, deviceId));
+    if(gpu_arch_match(deviceProperties.gcnArchName, "11\\d{2}"))
+        return EXIT_SUCCESS;
 
     // when arguments not specified, set to default values
     for(int i = 0; i < gemm_count; i++)
