@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -57,7 +57,8 @@ template <typename InTypeA,
           typename InTypeB,
           typename OutType,
           typename AlphaType,
-          typename BetaType>
+          typename BetaType,
+          typename BiasType = OutType>
 struct Runner
 {
     Runner(int64_t   m,
@@ -117,6 +118,40 @@ struct Runner
         CHECK_HIP_ERROR(hipFree(d_alphaVec));
         CHECK_HIPBLASLT_ERROR(hipblasLtDestroy(handle));
         CHECK_HIP_ERROR(hipStreamDestroy(stream));
+
+        if(biasVec)
+        {
+            CHECK_HIP_ERROR(hipFree(biasVec));
+            CHECK_HIP_ERROR(hipFree(d_biasVec));
+        }
+    }
+
+    void setBiasInfo(bool useBias, char biasSrc)
+    {
+        biasElems = 0;
+        if(useBias)
+        {
+            if(biasSrc == 'B' || biasSrc == 'b')
+                biasElems = n;
+            else if(biasSrc == 'A' || biasSrc == 'a' || biasSrc == 'D' || biasSrc == 'd')
+                biasElems = m;
+            // else, biasElems = 0
+        }
+
+        // alloc bias if use bias
+        if(biasElems > 0)
+        {
+            if(biasVec)
+            {
+                CHECK_HIP_ERROR(hipFree(biasVec));
+                CHECK_HIP_ERROR(hipFree(d_biasVec));
+            }
+
+            CHECK_HIP_ERROR(hipMalloc(&d_biasVec, biasElems * sizeof(BiasType)));
+            CHECK_HIP_ERROR(hipHostMalloc(&biasVec, biasElems * sizeof(BiasType)));
+            for(int i = 0; i < biasElems; ++i)
+                ((BiasType*)biasVec)[i] = static_cast<BiasType>((rand() % 7) - 3);
+        }
     }
 
     void hostToDevice()
@@ -129,6 +164,11 @@ struct Runner
             d_c, c, m * n * batch_count * sizeof(OutType), hipMemcpyHostToDevice, stream));
         CHECK_HIP_ERROR(hipMemcpyAsync(
             d_alphaVec, alphaVec, m * batch_count * sizeof(float), hipMemcpyHostToDevice, stream));
+
+        // cpy bias if needed
+        if(biasVec)
+            CHECK_HIP_ERROR(hipMemcpyAsync(
+                d_biasVec, biasVec, biasElems * sizeof(BiasType), hipMemcpyHostToDevice, stream));
     }
 
     void deviceToHost()
@@ -144,7 +184,7 @@ struct Runner
         static_cast<void>(func());
 
         deviceToHost();
-        hipStreamSynchronize(stream);
+        static_cast<void>(hipStreamSynchronize(stream));
     }
 
     int64_t   m;
@@ -160,6 +200,10 @@ struct Runner
     void*   d_workspace;
     int64_t max_workspace_size;
 
+    int64_t biasElems = 0;
+    void*   biasVec   = nullptr; // host
+    void*   d_biasVec = nullptr; // device
+
     hipStream_t       stream;
     hipblasLtHandle_t handle;
 };
@@ -291,7 +335,7 @@ struct RunnerVec
         static_cast<void>(func());
 
         deviceToHost();
-        hipStreamSynchronize(stream);
+        static_cast<void>(hipStreamSynchronize(stream));
     }
 
     std::vector<int64_t>   m;
@@ -315,24 +359,25 @@ template <typename Type>
 struct LayerNormRunner
 {
     LayerNormRunner(int64_t m, int64_t n)
-        : m(m), n(n)
+        : m(m)
+        , n(n)
     {
         CHECK_HIP_ERROR(hipStreamCreate(&stream));
         CHECK_HIPBLASLT_ERROR(hipblasLtCreate(&handle));
 
-        CHECK_HIP_ERROR(hipMalloc(&d_out,    m * n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipMalloc(&d_mean,   m * sizeof(Type)));
+        CHECK_HIP_ERROR(hipMalloc(&d_out, m * n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipMalloc(&d_mean, m * sizeof(Type)));
         CHECK_HIP_ERROR(hipMalloc(&d_invvar, m * sizeof(Type)));
-        CHECK_HIP_ERROR(hipMalloc(&d_in,     m * n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipMalloc(&d_gamma,  n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipMalloc(&d_beta,   n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipMalloc(&d_in, m * n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipMalloc(&d_gamma, n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipMalloc(&d_beta, n * sizeof(Type)));
 
-        CHECK_HIP_ERROR(hipHostMalloc(&out,    m * n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipHostMalloc(&mean,   m * sizeof(Type)));
+        CHECK_HIP_ERROR(hipHostMalloc(&out, m * n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipHostMalloc(&mean, m * sizeof(Type)));
         CHECK_HIP_ERROR(hipHostMalloc(&invvar, m * sizeof(Type)));
-        CHECK_HIP_ERROR(hipHostMalloc(&in,     m * n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipHostMalloc(&gamma,  n * sizeof(Type)));
-        CHECK_HIP_ERROR(hipHostMalloc(&beta,   n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipHostMalloc(&in, m * n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipHostMalloc(&gamma, n * sizeof(Type)));
+        CHECK_HIP_ERROR(hipHostMalloc(&beta, n * sizeof(Type)));
 
         for(int i = 0; i < m * n; i++)
             ((Type*)in)[i] = static_cast<Type>((rand() % 7) - 3);
@@ -364,22 +409,22 @@ struct LayerNormRunner
 
     void hostToDevice()
     {
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            d_in, in, m * n * sizeof(Type), hipMemcpyHostToDevice, stream));
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            d_gamma, gamma, n * sizeof(Type), hipMemcpyHostToDevice, stream));
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            d_beta, beta, n * sizeof(Type), hipMemcpyHostToDevice, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(d_in, in, m * n * sizeof(Type), hipMemcpyHostToDevice, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(d_gamma, gamma, n * sizeof(Type), hipMemcpyHostToDevice, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(d_beta, beta, n * sizeof(Type), hipMemcpyHostToDevice, stream));
     }
 
     void deviceToHost()
     {
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            out, d_out, m * n * sizeof(Type), hipMemcpyDeviceToHost, stream));
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            mean, d_mean, m * sizeof(Type), hipMemcpyDeviceToHost, stream));
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            invvar, d_invvar, m * sizeof(Type), hipMemcpyDeviceToHost, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(out, d_out, m * n * sizeof(Type), hipMemcpyDeviceToHost, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(mean, d_mean, m * sizeof(Type), hipMemcpyDeviceToHost, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(invvar, d_invvar, m * sizeof(Type), hipMemcpyDeviceToHost, stream));
     }
 
     void run(const std::function<void()>& func)
@@ -389,11 +434,11 @@ struct LayerNormRunner
         static_cast<void>(func());
 
         deviceToHost();
-        hipStreamSynchronize(stream);
+        static_cast<void>(hipStreamSynchronize(stream));
     }
 
-    int64_t   m;
-    int64_t   n;
+    int64_t m;
+    int64_t n;
 
     void *out, *mean, *invvar, *in, *gamma, *beta; // host
     void *d_out, *d_mean, *d_invvar, *d_in, *d_gamma, *d_beta; // host
@@ -406,7 +451,8 @@ template <typename Type>
 struct OptAMaxRunner
 {
     OptAMaxRunner(int64_t m, int64_t n)
-        : m(m), n(n)
+        : m(m)
+        , n(n)
     {
         CHECK_HIP_ERROR(hipStreamCreate(&stream));
         CHECK_HIPBLASLT_ERROR(hipblasLtCreate(&handle));
@@ -435,14 +481,13 @@ struct OptAMaxRunner
 
     void hostToDevice()
     {
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            d_in, in, m * n * sizeof(Type), hipMemcpyHostToDevice, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(d_in, in, m * n * sizeof(Type), hipMemcpyHostToDevice, stream));
     }
 
     void deviceToHost()
     {
-        CHECK_HIP_ERROR(hipMemcpyAsync(
-            out, d_out, sizeof(Type), hipMemcpyDeviceToHost, stream));
+        CHECK_HIP_ERROR(hipMemcpyAsync(out, d_out, sizeof(Type), hipMemcpyDeviceToHost, stream));
     }
 
     void run(const std::function<void()>& func)
@@ -452,11 +497,11 @@ struct OptAMaxRunner
         static_cast<void>(func());
 
         deviceToHost();
-        hipStreamSynchronize(stream);
+        static_cast<void>(hipStreamSynchronize(stream));
     }
 
-    int64_t   m;
-    int64_t   n;
+    int64_t m;
+    int64_t n;
 
     void *in, *out; // host
     void *d_in, *d_out; // device
@@ -464,3 +509,78 @@ struct OptAMaxRunner
     hipStream_t       stream;
     hipblasLtHandle_t handle;
 };
+
+template <typename T, typename Ts>
+struct OptAMaxWithScaleRunner
+{
+    OptAMaxWithScaleRunner(int64_t m, int64_t n)
+        : m(m)
+        , n(n)
+    {
+        CHECK_HIP_ERROR(hipStreamCreate(&stream));
+        CHECK_HIPBLASLT_ERROR(hipblasLtCreate(&handle));
+
+        CHECK_HIP_ERROR(hipMalloc(&d_out, sizeof(T)));
+        CHECK_HIP_ERROR(hipMalloc(&d_outD, m * n * sizeof(Ts)));
+        CHECK_HIP_ERROR(hipMalloc(&d_in, m * n * sizeof(T)));
+        CHECK_HIP_ERROR(hipMalloc(&d_in_scale, 1 * sizeof(float)));
+
+        CHECK_HIP_ERROR(hipHostMalloc(&out, sizeof(T)));
+        CHECK_HIP_ERROR(hipHostMalloc(&outD, m * n * sizeof(Ts)));
+        CHECK_HIP_ERROR(hipHostMalloc(&in, m * n * sizeof(T)));
+        CHECK_HIP_ERROR(hipHostMalloc(&in_scale, 1 * sizeof(float)));
+
+        for(int i = 0; i < m * n; i++)
+            ((T*)in)[i] = static_cast<T>((rand() % 7) - 3);
+        *(float*)in_scale = static_cast<float>(0.5);
+    }
+
+    ~OptAMaxWithScaleRunner()
+    {
+        CHECK_HIP_ERROR(hipFree(d_out));
+        CHECK_HIP_ERROR(hipFree(d_outD));
+        CHECK_HIP_ERROR(hipFree(d_in));
+        CHECK_HIP_ERROR(hipFree(d_in_scale));
+
+        CHECK_HIP_ERROR(hipFree(out));
+        CHECK_HIP_ERROR(hipFree(outD));
+        CHECK_HIP_ERROR(hipFree(in));
+        CHECK_HIP_ERROR(hipFree(in_scale));
+
+        CHECK_HIPBLASLT_ERROR(hipblasLtDestroy(handle));
+        CHECK_HIP_ERROR(hipStreamDestroy(stream));
+    }
+
+    void hostToDevice()
+    {
+        CHECK_HIP_ERROR(hipMemcpyAsync(d_in, in, m * n * sizeof(T), hipMemcpyHostToDevice, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(d_in_scale, in_scale, 1 * sizeof(float), hipMemcpyHostToDevice, stream));
+    }
+
+    void deviceToHost()
+    {
+        CHECK_HIP_ERROR(hipMemcpyAsync(out, d_out, sizeof(T), hipMemcpyDeviceToHost, stream));
+        CHECK_HIP_ERROR(
+            hipMemcpyAsync(outD, d_outD, m * n * sizeof(Ts), hipMemcpyDeviceToHost, stream));
+    }
+
+    void run(const std::function<void()>& func)
+    {
+        hostToDevice();
+
+        static_cast<void>(func());
+
+        deviceToHost();
+        static_cast<void>(hipStreamSynchronize(stream));
+    }
+
+    int64_t m;
+    int64_t n;
+
+    void *in, *in_scale, *out, *outD; // host
+    void *d_in, *d_in_scale, *d_out, *d_outD; // device
+
+    hipStream_t       stream;
+    hipblasLtHandle_t handle;
+};
