@@ -135,6 +135,7 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
     void*              scaleC        = matmul_descr->scaleC;
     void*              scaleD        = matmul_descr->scaleD;
     void*              scaleE        = matmul_descr->scaleE;
+    void*              amaxD         = matmul_descr->amaxD;
 
     // Others
     constexpr bool strided_batch = true;
@@ -189,8 +190,11 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
                                         scaleD,
                                         scaleE,
                                         scaleAlphaVec,
+                                        matmul_descr->isScaleAVec,
+                                        matmul_descr->isScaleBVec,
                                         bias_type,
                                         epilogue,
+                                        amaxD,
                                         nullptr,
                                         maxWorkSpaceBytes,
                                         nullptr,
@@ -552,6 +556,17 @@ rocblaslt_status rocblaslt_matmul_desc_create(rocblaslt_matmul_desc* matmulDesc,
             case rocblaslt_compute_f64:
             case rocblaslt_compute_i32:
             case rocblaslt_compute_f32_fast_f16:
+            case rocblaslt_compute_f32_fast_bf16:
+            case rocblaslt_compute_f32_fast_f8_fnuz:
+            case rocblaslt_compute_f32_fast_bf8_fnuz:
+            case rocblaslt_compute_f32_fast_f8bf8_fnuz:
+            case rocblaslt_compute_f32_fast_bf8f8_fnuz:
+#ifdef ROCM_USE_FLOAT8
+            case rocblaslt_compute_f32_fast_f8_ocp:
+            case rocblaslt_compute_f32_fast_bf8_ocp:
+            case rocblaslt_compute_f32_fast_f8bf8_ocp:
+            case rocblaslt_compute_f32_fast_bf8f8_ocp:
+#endif
                 break;
             default:
                 log_error(__func__, "invalid compute type", computeType);
@@ -564,13 +579,15 @@ rocblaslt_status rocblaslt_matmul_desc_create(rocblaslt_matmul_desc* matmulDesc,
                 throw rocblaslt_status_invalid_value;
             }
 
-            *matmulDesc                 = new _rocblaslt_matmul_desc();
-            (*matmulDesc)->compute_type = computeType;
-            (*matmulDesc)->scale_type   = scaleType;
-            auto computeTypeInit        = computeType == rocblaslt_compute_f32_fast_xf32
-                                              ? rocblaslt_compute_f32
-                                              : computeType;
-            auto dataType               = HIP_R_32F;
+            *matmulDesc = new _rocblaslt_matmul_desc();
+
+            (*matmulDesc)->compute_type          = computeType;
+            (*matmulDesc)->compute_type_original = computeType;
+            (*matmulDesc)->scale_type            = scaleType;
+            auto computeTypeInit                 = computeType == rocblaslt_compute_f32_fast_xf32
+                                                       ? rocblaslt_compute_f32
+                                                       : computeType;
+            auto dataType                        = HIP_R_32F;
             if(computeTypeInit == rocblaslt_compute_f64)
                 dataType = HIP_R_64F;
             else if(computeType == rocblaslt_compute_i32)
@@ -629,6 +646,38 @@ rocblaslt_status rocblaslt_matmul_desc_destroy(const rocblaslt_matmul_desc matmu
     return rocblaslt_status_success;
 }
 
+rocblaslt_compute_type _matmul_desc_determine_compute_type(rocblaslt_matmul_desc matmulDesc)
+{
+    if(matmulDesc->compute_type_original == rocblaslt_compute_f32)
+    {
+        auto tciA = matmulDesc->compute_input_typeA;
+        auto tciB = matmulDesc->compute_input_typeB;
+        if(tciA == tciB && tciA == HIP_R_16F)
+            return rocblaslt_compute_f32_fast_f16;
+        else if(tciA == tciB && tciA == HIP_R_16BF)
+            return rocblaslt_compute_f32_fast_bf16;
+        else if(tciA == tciB && tciA == HIP_R_8F_E4M3_FNUZ)
+            return rocblaslt_compute_f32_fast_f8_fnuz;
+        else if(tciA == tciB && tciA == HIP_R_8F_E5M2_FNUZ)
+            return rocblaslt_compute_f32_fast_bf8_fnuz;
+        else if(tciA == HIP_R_8F_E4M3_FNUZ && tciB == HIP_R_8F_E5M2_FNUZ)
+            return rocblaslt_compute_f32_fast_f8bf8_fnuz;
+        else if(tciA == HIP_R_8F_E5M2_FNUZ && tciB == HIP_R_8F_E4M3_FNUZ)
+            return rocblaslt_compute_f32_fast_bf8f8_fnuz;
+#ifdef ROCM_USE_FLOAT8
+        else if(tciA == tciB && tciA == HIP_R_8F_E4M3)
+            return rocblaslt_compute_f32_fast_f8_ocp;
+        else if(tciA == tciB && tciA == HIP_R_8F_E5M2)
+            return rocblaslt_compute_f32_fast_bf8_ocp;
+        else if(tciA == HIP_R_8F_E4M3 && tciB == HIP_R_8F_E5M2)
+            return rocblaslt_compute_f32_fast_f8bf8_ocp;
+        else if(tciA == HIP_R_8F_E5M2 && tciB == HIP_R_8F_E4M3)
+            return rocblaslt_compute_f32_fast_bf8f8_ocp;
+#endif
+    }
+    return matmulDesc->compute_type_original;
+}
+
 /********************************************************************************
  * \brief sets the value of the specified attribute belonging to matrix
  *multiplication descriptor.
@@ -697,7 +746,11 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
+            case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT:
+                matmulDesc->isScaleAVec = true;
             case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER:
+                if(matmulAttr == ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER)
+                    matmulDesc->isScaleAVec = false;
                 if(sizeof(void*) <= sizeInBytes)
                     memcpy(&matmulDesc->scaleA, buf, sizeof(void*));
                 else
@@ -706,7 +759,11 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
+            case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT:
+                matmulDesc->isScaleBVec = true;
             case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER:
+                if(matmulAttr == ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER)
+                    matmulDesc->isScaleBVec = false;
                 if(sizeof(void*) <= sizeInBytes)
                     memcpy(&matmulDesc->scaleB, buf, sizeof(void*));
                 else
@@ -787,6 +844,39 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
+            case ROCBLASLT_MATMUL_DESC_AMAX_D_POINTER:
+                if(sizeof(void*) <= sizeInBytes)
+                    memcpy(&matmulDesc->amaxD, buf, sizeof(void*));
+                else
+                {
+                    log_error(__func__, "invalid amax buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
+            case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT:
+                if(sizeof(int32_t) <= sizeInBytes)
+                {
+                    memcpy(&matmulDesc->compute_input_typeA, buf, sizeof(int32_t));
+                    matmulDesc->compute_type = _matmul_desc_determine_compute_type(matmulDesc);
+                }
+                else
+                {
+                    log_error(__func__, "invalid compute_input_typeA buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
+            case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT:
+                if(sizeof(int32_t) <= sizeInBytes)
+                {
+                    memcpy(&matmulDesc->compute_input_typeB, buf, sizeof(int32_t));
+                    matmulDesc->compute_type = _matmul_desc_determine_compute_type(matmulDesc);
+                }
+                else
+                {
+                    log_error(__func__, "invalid compute_input_typeB buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
             default:
                 log_error(__func__, "invalid attribute", matmulAttr);
                 return rocblaslt_status_invalid_value;
@@ -885,6 +975,7 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 memcpy(buf, &matmulDesc->bias, sizeof(void*));
                 break;
             case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER:
+            case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT:
                 if(sizeWritten)
                     *sizeWritten = sizeof(void*);
                 if(sizeInBytes < sizeof(void*))
@@ -895,6 +986,7 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 memcpy(buf, &matmulDesc->scaleA, sizeof(void*));
                 break;
             case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER:
+            case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT:
                 if(sizeWritten)
                     *sizeWritten = sizeof(void*);
                 if(sizeInBytes < sizeof(void*))
@@ -912,7 +1004,7 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                     log_error(__func__, "invalid buf size", sizeInBytes);
                     return rocblaslt_status_invalid_value;
                 }
-                memcpy(buf, &matmulDesc->pointermode, sizeof(void*));
+                memcpy(buf, &matmulDesc->pointermode, sizeof(int32_t));
                 break;
             case ROCBLASLT_MATMUL_DESC_BIAS_DATA_TYPE:
                 if(sizeWritten)
@@ -924,6 +1016,36 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 }
                 memcpy(buf, &matmulDesc->bias_type, sizeof(int32_t));
                 break;
+            case ROCBLASLT_MATMUL_DESC_AMAX_D_POINTER:
+                if(sizeWritten)
+                    *sizeWritten = sizeof(void*);
+                if(sizeInBytes < sizeof(void*))
+                {
+                    log_error(__func__, "invalid buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                memcpy(buf, &matmulDesc->amaxD, sizeof(void*));
+                break;
+            case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT:
+                if(sizeWritten)
+                    *sizeWritten = sizeof(int32_t);
+                if(sizeInBytes < sizeof(int32_t))
+                {
+                    log_error(__func__, "invalid buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                memcpy(buf, &matmulDesc->compute_input_typeA, sizeof(int32_t));
+                break;
+            case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT:
+                if(sizeWritten)
+                    *sizeWritten = sizeof(int32_t);
+                if(sizeInBytes < sizeof(int32_t))
+                {
+                    log_error(__func__, "invalid buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                memcpy(buf, &matmulDesc->compute_input_typeB, sizeof(int32_t));
+                break;
             default:
                 log_error(__func__, "invalid attribute", matmulAttr);
                 return rocblaslt_status_invalid_value;
@@ -1216,11 +1338,16 @@ rocblaslt_status
         hipDataType            d_type       = matD->type;
         rocblaslt_compute_type compute_type = matmul_desc->compute_type;
         auto&                  tensile_data = matmul_desc->m_data;
-
-        int8_t alpha[16] = {0};
-        int8_t beta[16]  = {0};
+        int8_t                 alpha[16]    = {0};
+        int8_t                 beta[16]     = {0};
         assignAlphaBeta1(compute_type, (void*)alpha, (void*)beta);
-
+        //bias ptr can be set later after getting solution.
+        bool dummy_bias_address = false;
+        if(matmul_desc->bias == nullptr && is_bias_enabled(matmul_desc->epilogue))
+        {
+            dummy_bias_address = true;
+            matmul_desc->bias  = &dummy_bias_address;
+        }
         auto prob = construct_rocblaslt_problem(
             handle, matmul_desc, matA, matB, matC, matD, &alpha, &beta, pref->max_workspace_bytes);
         status = getBestSolutions(prob,
@@ -1230,8 +1357,53 @@ rocblaslt_status
                                   heuristicResultsArray,
                                   returnAlgoCount,
                                   pref->max_workspace_bytes);
-
+        if(dummy_bias_address)
+            matmul_desc->bias = nullptr;
         log_api(__func__, "returnAlogCount", *returnAlgoCount);
+
+        //Try to get size independent solutions from getAllSolutions()
+        if(requestedAlgoCount > *returnAlgoCount)
+        {
+            std::vector<rocblaslt_matmul_heuristic_result> allSolutionsResults;
+            if(rocblaslt_status_success
+               == getAllSolutions(prob, handle, allSolutionsResults, pref->max_workspace_bytes))
+            {
+                int oriReturnAlgoCount = *returnAlgoCount;
+                for(int i = 0;
+                    *returnAlgoCount < requestedAlgoCount && i < allSolutionsResults.size();
+                    i++)
+                {
+                    bool   duplicated_sol          = false;
+                    size_t required_workspace_size = 0;
+                    for(int j = 0; j < oriReturnAlgoCount; j++)
+                        if(*(int*)(heuristicResultsArray[j].algo.data)
+                           == *(int*)(allSolutionsResults[i].algo.data)) //solution index
+                            duplicated_sol = true;
+
+                    if(duplicated_sol == true
+                       || rocblaslt_status_success
+                              != isSolutionSupported(handle,
+                                                     prob,
+                                                     tensile_data,
+                                                     &allSolutionsResults[i].algo,
+                                                     &required_workspace_size))
+                        continue;
+                    //append sol to heuristpicResultsArray
+                    memcpy(heuristicResultsArray[*returnAlgoCount].algo.data,
+                           allSolutionsResults[i].algo.data,
+                           sizeof(heuristicResultsArray[i].algo.data));
+                    heuristicResultsArray[*returnAlgoCount].algo.max_workspace_bytes
+                        = pref->max_workspace_bytes;
+                    heuristicResultsArray[*returnAlgoCount].algo.fallback = false;
+                    heuristicResultsArray[*returnAlgoCount].state = rocblaslt_status_success;
+                    heuristicResultsArray[*returnAlgoCount].workspaceSize = required_workspace_size;
+                    (*returnAlgoCount)++;
+                }
+
+                log_api(__func__, "final returnAlogCount", *returnAlgoCount);
+            }
+        }
+
         if(status != rocblaslt_status_success)
         {
             throw status;
@@ -1359,10 +1531,7 @@ rocblaslt_status rocblaslt_matmul_get_algos_from_index_cpp(
         status = getSolutionsFromIndex(handle, solutionIndex, heuristicResults, maxWorkspaceSize);
 
         log_api(__func__, "returnAlogCount", heuristicResults.size());
-        if(status != rocblaslt_status_success)
-        {
-            throw status;
-        }
+        return status;
     }
     catch(const rocblaslt_status& status)
     {
@@ -1381,6 +1550,16 @@ rocblaslt_status rocblaslt_is_algo_supported_cpp(rocblaslt_handle            han
     return isSolutionSupported(handle, gemmType, gemmData, algo, tuning, workspaceSizeInBytes);
 }
 
+rocblaslt_status rocblaslt_is_algo_supported_cpp(rocblaslt_handle              handle,
+                                                 rocblaslt::RocGemmType        gemmType,
+                                                 std::shared_ptr<void>         gemmData,
+                                                 rocblaslt_matmul_algo&        algo,
+                                                 const rocblaslt::RocTuningV2* tuning,
+                                                 size_t&                       workspaceSizeInBytes)
+{
+    return isSolutionSupported(handle, gemmType, gemmData, algo, tuning, workspaceSizeInBytes);
+}
+
 rocblaslt_status
     rocblaslt_algo_get_heuristic_cpp(rocblaslt_handle       handle,
                                      rocblaslt::RocGemmType gemmType,
@@ -1396,7 +1575,9 @@ rocblaslt_status
     }
     if(gemmType == rocblaslt::RocGemmType::ROCBLASLT_GROUPED_GEMM)
     {
-        log_api(__func__, "will be deprecated for groupedgemm in the future, please use get_all_algos instead");
+        log_api(
+            __func__,
+            "will be deprecated for groupedgemm in the future, please use get_all_algos instead");
     }
     rocblaslt_status status = rocblaslt_status_success;
     try
@@ -1409,6 +1590,43 @@ rocblaslt_status
         {
             throw status;
         }
+        //Try to get size independent solutions from getAllSolutions()
+        if(requestedAlgoCount > results.size())
+        {
+            std::vector<rocblaslt_matmul_heuristic_result> allSolutionsResults;
+            size_t                                         workspaceSizeInBytes = workspaceBytes;
+            if(rocblaslt_status_success
+               == getAllSolutions(
+                   gemmData, handle, gemmType, allSolutionsResults, workspaceSizeInBytes))
+            {
+                int oriReturnAlgoCount = results.size();
+                for(int i = 0;
+                    results.size() < requestedAlgoCount && i < allSolutionsResults.size();
+                    i++)
+                {
+                    bool duplicated_sol = false;
+                    for(int j = 0; j < oriReturnAlgoCount; j++)
+                        if(*(int*)(results[j].algo.data)
+                           == *(int*)(allSolutionsResults[i].algo.data)) //solution index
+                            duplicated_sol = true;
+                    rocblaslt::RocTuningV2* tuning = nullptr;
+                    if(duplicated_sol == true
+                       || rocblaslt_status_success
+                              != isSolutionSupported(
+                                  handle,
+                                  static_cast<const rocblaslt::RocGemmType>(gemmType),
+                                  gemmData,
+                                  allSolutionsResults[i].algo,
+                                  tuning,
+                                  workspaceSizeInBytes))
+                        continue;
+                    allSolutionsResults[i].workspaceSize = workspaceSizeInBytes;
+                    results.push_back(allSolutionsResults[i]);
+                }
+
+                log_api(__func__, "final returnAlogCount", results.size());
+            }
+        }
     }
     catch(const rocblaslt_status& status)
     {
@@ -1494,3 +1712,41 @@ void rocblaslt_log_error(const char* func, const char* var, const char* msg)
 {
     log_error(func, var, msg);
 }
+
+extern "C" int rocblaslt_matmul_is_tuned(rocblaslt_handle        handle,
+                                         rocblaslt_matmul_desc   matmul_descr,
+                                         rocblaslt_matrix_layout matA,
+                                         rocblaslt_matrix_layout matB,
+                                         rocblaslt_matrix_layout matC,
+                                         rocblaslt_matrix_layout matD)
+{
+    if(handle == nullptr || matmul_descr == nullptr || matA == nullptr || matB == nullptr
+       || matC == nullptr || matD == nullptr)
+    {
+        log_error(__func__, "invalid handle pointer");
+        return -1;
+    }
+
+    hipDataType            a_type              = matA->type;
+    hipDataType            b_type              = matB->type;
+    hipDataType            c_type              = matC->type;
+    hipDataType            d_type              = matD->type;
+    rocblaslt_compute_type compute_type        = matmul_descr->compute_type;
+    auto&                  gemmData            = matmul_descr->m_data;
+    float                  alpha               = 1.f;
+    float                  beta                = 0.f;
+    constexpr size_t       max_workspace_bytes = 32 * 1024 * 1024;
+    void*                  alphaf              = &alpha;
+    void*                  betaf               = &beta;
+    auto                   prob                = construct_rocblaslt_problem(
+        handle, matmul_descr, matA, matB, matC, matD, alphaf, betaf, max_workspace_bytes);
+    auto& tensile_data = matmul_descr->m_data;
+    auto  sols         = getBestRawSolutions(prob, handle, tensile_data, 1, max_workspace_bytes);
+
+    if(sols.size() && sols.front()->tag == Tensile::ContractionSolution::MatchingTag::Equal)
+    {
+        return 1;
+    }
+
+    return 0;
+}
