@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -33,7 +33,7 @@ from . import ClientExecutable
 from . import SolutionLibrary
 from . import LibraryIO
 from . import Utils
-from .BenchmarkStructs import BenchmarkProcess, checkParametersAreValid, constructForkPermutations
+from .BenchmarkStructs import BenchmarkProcess, constructForkPermutations
 from .Contractions import ProblemType as ContractionsProblemType
 from .ClientWriter import runClient, writeClientConfig, writeClientConfigIni
 from .Common import globalParameters, HR, pushWorkingPath, popWorkingPath, print1, print2, \
@@ -69,20 +69,7 @@ def generateForkedSolutions(problemType, constantParams, forkPermutations):
 
 def getCustomKernelSolutionObj(kernelName, internalSupportParams, directory=globalParameters["CustomKernelDirectory"]):
     """Creates the Solution object for a custom kernel"""
-    kernelConfig = getCustomKernelConfig(kernelName, directory)
-    if "InternalSupportParams" not in kernelConfig:
-        kernelConfig["InternalSupportParams"] = internalSupportParams
-    else:
-        for key in internalSupportParams:
-            if key not in kernelConfig["InternalSupportParams"]:
-                kernelConfig["InternalSupportParams"][key] = internalSupportParams[key]
-    for k, v in kernelConfig.items():
-        if k != "ProblemType":
-            checkParametersAreValid((k, [v]), validParameters)
-    kernelConfig["KernelLanguage"] = "Assembly"
-    kernelConfig["CustomKernelName"] = kernelName
-
-    return Solution(kernelConfig)
+    return Solution(getCustomKernelConfig(kernelName, internalSupportParams, directory))
 
 
 def generateCustomKernelSolutions(problemType, customKernels, internalSupportParams, failOnMismatch):
@@ -119,7 +106,7 @@ def generateCustomKernelSolutions(problemType, customKernels, internalSupportPar
     return solutions
 
 def writeBenchmarkFiles(stepBaseDir, solutions, problemSizes, \
-        biasTypeArgs, activationArgs, stepName, solutionSummationSizes):
+        biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, stepName, solutionSummationSizes):
     """Write all the files needed for a given benchmarking step"""
     if not globalParameters["MergeFiles"]:
         ensurePath(os.path.join(globalParameters["WorkingPath"], "Solutions"))
@@ -191,10 +178,10 @@ def writeBenchmarkFiles(stepBaseDir, solutions, problemSizes, \
                 idealSize = {"Exact": [idealM, idealN, idealK]}
                 idealSizes.append(idealSize)
         idealProblemSizes = ProblemSizes(problemType, idealSizes)
-        writeClientConfig(True, solutions, idealProblemSizes, biasTypeArgs, activationArgs, stepName, stepBaseDir, \
+        writeClientConfig(True, solutions, idealProblemSizes, biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, stepName, stepBaseDir, \
             newLibrary, codeObjectFiles, True)
     else:
-        writeClientConfig(True, solutions, problemSizes, biasTypeArgs, activationArgs, stepName, stepBaseDir, \
+        writeClientConfig(True, solutions, problemSizes, biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, stepName, stepBaseDir, \
             newLibrary, codeObjectFiles, False)
 
     if len(solutions) == 0:
@@ -239,8 +226,10 @@ def benchmarkProblemType(problemTypeConfig, problemSizeGroupConfig, problemSizeG
         elapsedTime = currentTime - startTime
         print1("# Benchmark Step: {} - {} {:.3f}s".format(groupName, stepName, elapsedTime))
         print1("# Num Sizes: {}".format(benchmarkStep.problemSizes.totalProblemSizes))
-        print1("# Activation steps: {}".format(benchmarkStep.biasTypeArgs.totalProblemSizes))
+        print1("# Factor Dim steps: {}".format(benchmarkStep.factorDimArgs.totalProblemSizes))
+        print1("# Bias Type steps: {}".format(benchmarkStep.biasTypeArgs.totalProblemSizes))
         print1("# Activation steps: {}".format(benchmarkStep.activationArgs.totalProblemSizes))
+        print1("# ICacheFlush steps: {}".format(len(benchmarkStep.icacheFlushArgs)))
         print1("# Fork Parameters:")
         for k, v in benchmarkStep.forkParams.items():
             print1("#     {}: {}".format(k, v))
@@ -264,7 +253,7 @@ def benchmarkProblemType(problemTypeConfig, problemSizeGroupConfig, problemSizeG
 
         cacheValid = False
         if useCache and os.path.isfile(cachePath):
-            c = LibraryIO.readYAML(cachePath)
+            c = LibraryIO.read(cachePath)
             if c["ConstantParams"] == benchmarkStep.constantParams and \
                     c["ForkParams"] == benchmarkStep.forkParams and \
                     c["ParamGroups"] == benchmarkStep.paramGroups and \
@@ -312,9 +301,10 @@ def benchmarkProblemType(problemTypeConfig, problemSizeGroupConfig, problemSizeG
 
             # write benchmarkFiles
             prevCount = len(solutions)
-            codeObjectFiles = writeBenchmarkFiles(stepBaseDir, solutions, \
-                    benchmarkStep.problemSizes, benchmarkStep.biasTypeArgs, \
-                    benchmarkStep.activationArgs, shortName, [])
+            codeObjectFiles = writeBenchmarkFiles(stepBaseDir, solutions,      \
+                    benchmarkStep.problemSizes, benchmarkStep.biasTypeArgs,    \
+                    benchmarkStep.factorDimArgs, benchmarkStep.activationArgs, \
+                    benchmarkStep.icacheFlushArgs, shortName, [])
             # ^ this mutates solutions
 
             # write cache data
@@ -347,7 +337,8 @@ def benchmarkProblemType(problemTypeConfig, problemSizeGroupConfig, problemSizeG
             outFile = os.path.join(globalParameters["WorkingPath"], "ClientParameters.ini")
 
             writeClientConfigIni(benchmarkStep.problemSizes, benchmarkStep.biasTypeArgs,
-                                 benchmarkStep.activationArgs, conProblemType,
+                                 benchmarkStep.factorDimArgs, benchmarkStep.activationArgs,
+                                 benchmakrStep.icacheFlushArgs, conProblemType,
                                  globalParameters["WorkingPath"], codeObjectFiles, resultsFileName,
                                  outFile)
 
@@ -386,6 +377,10 @@ def main(config, useCache):
     """Entry point for the "BenchmarkProblems" section of a Tensile config yaml"""
     ClientExecutable.getClientExecutable()
 
+    if config is None:
+        print(f'No config specified in {globalParameters["ConfigPath"]}, built client only')
+        return
+
     dataPath = os.path.join(globalParameters["WorkingPath"], globalParameters["BenchmarkDataPath"])
     pushWorkingPath(globalParameters["BenchmarkProblemsPath"])
     ensurePath(dataPath)
