@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,8 @@
 #define HIP_HOST __host__
 #define HIP_DEVICE __device__
 
+#include <hip/hip_fp8.h>
+
 namespace tensile_hip_f8_impl
 {
 
@@ -113,6 +115,35 @@ namespace Tensile
 #endif
     }
 
+    static bool        isOcpF8;
+    static inline bool IsOCPSupported()
+    {
+        static bool     do_once = false;
+        int             deviceId;
+        hipDeviceProp_t deviceProperties;
+        auto            removePrefix = [](const std::string& s) {
+            size_t pos = s.find("gfx");
+            if(pos != std::string::npos)
+            {
+                return s.substr(pos + 3);
+            }
+            return s;
+        };
+
+        if(!do_once)
+        {
+            static_cast<void>(hipGetDevice(&deviceId));
+            static_cast<void>(hipGetDeviceProperties(&deviceProperties, deviceId));
+            auto gpu_arch_no_prefix = removePrefix(deviceProperties.gcnArchName);
+            if(stoi(gpu_arch_no_prefix) / 100 == 12)
+                isOcpF8 = true;
+            else
+                isOcpF8 = false;
+            do_once = true;
+        }
+        return isOcpF8;
+    }
+
     // data type
     template <hip_f8_type T>
     struct Float8_BFloat8
@@ -189,6 +220,68 @@ namespace Tensile
             }
         }
 
+        // only host code is simulated
+        explicit HIP_HOST
+#elif defined(__gfx1200__)
+        // constructor from float using s/w simulation
+        // Device implementation using intrinsic code
+        explicit HIP_DEVICE Float8_BFloat8(float                v,
+                                           hip_f8_rounding_mode rm = hip_f8_rounding_mode::standard,
+                                           uint32_t             rng = 0)
+        {
+            union
+            {
+                float    fval;
+                uint32_t i32val;
+                uint8_t  i8val[4]; // not endian independent
+            } val;
+
+            uint32_t ival = 0;
+            val.fval      = v;
+
+            if(T == hip_f8_type::bf8)
+            {
+                // add clipping code.. by default, always clipping for now
+                if((val.i32val & 0x7F800000) != 0x7F800000) // all exp bits  are 1 --> NaN or INF
+                    val.fval = __builtin_amdgcn_fmed3f(val.fval, 57344.0, -57344.0);
+
+                // TODO: make it compile-time
+                if(rm == hip_f8_rounding_mode::stochastic)
+                {
+                    ival       = __builtin_amdgcn_cvt_sr_bf8_f32(val.fval, rng, ival, 0); // 0 pos
+                    val.i32val = ival;
+                    data       = val.i8val[0]; // little endian
+                }
+                else // RNE CVT
+                {
+                    ival = __builtin_amdgcn_cvt_pk_bf8_f32(
+                        val.fval, val.fval, ival, false); // false -> WORD0
+                    val.i32val = ival;
+                    data       = val.i8val[0];
+                }
+            }
+            else // fp8
+            {
+                if((val.i32val & 0x7F800000) != 0x7F800000) // all exp bits  are 1 --> NaN or INF
+                    val.fval = __builtin_amdgcn_fmed3f(val.fval, 448.0, -448.0);
+
+                // TODO: make this if-statement compile-time
+                if(rm == hip_f8_rounding_mode::stochastic)
+                {
+                    ival       = __builtin_amdgcn_cvt_sr_fp8_f32(val.fval, rng, ival, 0); // 0 pos
+                    val.i32val = ival;
+                    data       = val.i8val[0]; // little endian
+                }
+                else // RNE CVT
+                {
+                    ival = __builtin_amdgcn_cvt_pk_fp8_f32(
+                        val.fval, val.fval, ival, false); // false -> WORD0
+                    val.i32val = ival;
+                    data       = val.i8val[0];
+                }
+            }
+        }
+
         // only host code is simulated
         explicit HIP_HOST
 #else // gfx940
@@ -198,35 +291,52 @@ namespace Tensile
                            hip_f8_rounding_mode rm  = hip_f8_rounding_mode::standard,
                            uint32_t             rng = 0)
         {
+            assert(T == hip_f8_type::fp8 || T == hip_f8_type::bf8);
             // NOTE: made clipping default again
             if(T == hip_f8_type::bf8)
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<2, 5, float, true /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    data = internal::cast_to_f8<float, false /*is_funz*/>(
+                        v, 2, 5, true /*clip*/, (rm == hip_f8_rounding_mode::stochastic), rng);
                 else
+#endif
                 {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<2, 5, float, false /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<2, 5, float, true /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
+                    else
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<2, 5, float, false /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
                 }
             }
             else /* fp8*/
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<3, 4, float, true /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    data = internal::cast_to_f8<float, false /*is_funz*/>(
+                        v, 3, 4, true /*clip*/, (rm == hip_f8_rounding_mode::stochastic), rng);
                 else
+#endif
                 {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<3, 4, float, false /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<3, 4, float, true /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
+                    else
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<3, 4, float, false /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
                 }
             }
         }
@@ -241,6 +351,16 @@ namespace Tensile
         {
         }
 
+        explicit HIP_HOST
+#elif defined(__gfx1200__)
+        // convert double to f32 and call constructor
+        explicit HIP_DEVICE Float8_BFloat8(double               v,
+                                           hip_f8_rounding_mode rm = hip_f8_rounding_mode::standard,
+                                           uint32_t             rng = 0)
+            : Float8_BFloat8(static_cast<float>(v), rm, rng)
+        {
+        }
+
         explicit HIP_HOST
 #else
         explicit HIP_HOST_DEVICE
@@ -262,6 +382,16 @@ namespace Tensile
         {
         }
 
+        explicit HIP_HOST
+#elif defined(__gfx1200__)
+        // no h/w inst for cvt from f16, just convert f16 to f32 and call constructor
+        explicit HIP_DEVICE Float8_BFloat8(_Float16             v,
+                                           hip_f8_rounding_mode rm = hip_f8_rounding_mode::standard,
+                                           uint32_t             rng = 0)
+            : Float8_BFloat8((float)v, rm, rng)
+        {
+        }
+
         explicit HIP_HOST
 #else
         explicit HIP_HOST_DEVICE
@@ -270,35 +400,52 @@ namespace Tensile
                            hip_f8_rounding_mode rm  = hip_f8_rounding_mode::standard,
                            uint32_t             rng = 0)
         {
+            assert(T == hip_f8_type::fp8 || T == hip_f8_type::bf8);
             // NOTE: made clipping default again
             if(T == hip_f8_type::bf8)
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<2, 5, _Float16, true /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    data = internal::cast_to_f8<float, false /*is_funz*/>(
+                        v, 2, 5, true /*clip*/, (rm == hip_f8_rounding_mode::stochastic), rng);
                 else
+#endif
                 {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<2, 5, _Float16, false /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<2, 5, _Float16, true /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
+                    else
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<2, 5, _Float16, false /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
                 }
             }
             else /* fp8*/
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<3, 4, _Float16, true /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    data = internal::cast_to_f8<float, false /*is_funz*/>(
+                        v, 3, 4, true /*clip*/, (rm == hip_f8_rounding_mode::stochastic), rng);
                 else
+#endif
                 {
-                    data = tensile_hip_f8_impl::
-                        cast_to_f8<3, 4, _Float16, false /*negative_zero_nan*/, true /*clip*/>(
-                            v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<3, 4, _Float16, true /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
+                    else
+                    {
+                        data = tensile_hip_f8_impl::
+                            cast_to_f8<3, 4, _Float16, false /*negative_zero_nan*/, true /*clip*/>(
+                                v, (rm == hip_f8_rounding_mode::stochastic), rng);
+                    }
                 }
             }
         }
@@ -341,36 +488,67 @@ namespace Tensile
             return fval;
         }
         explicit inline HIP_HOST operator float() const
-
+#elif defined(__gfx1200__)
+        // builtin conversion
+        explicit inline HIP_DEVICE operator float() const
+        {
+            float    fval;
+            uint32_t i32val = static_cast<uint32_t>(data);
+            if(T == hip_f8_type::bf8)
+                // workaround: use inline asm instead of builtin function
+                fval = __builtin_amdgcn_cvt_f32_bf8(i32val, 0);
+            //asm volatile("v_cvt_f32_bf8 %0, %1 src0_sel:BYTE_0" : "=v"(fval) : "v"(i32val));
+            else
+                // workaround: use inline asm instead of builtin function
+                fval = __builtin_amdgcn_cvt_f32_fp8(i32val, 0);
+            //asm volatile("v_cvt_f32_fp8 %0, %1 src0_sel:BYTE_0" : "=v"(fval) : "v"(i32val));
+            return fval;
+        }
+        explicit inline HIP_HOST operator float() const
 #else // non gfx940
 
         explicit inline HIP_HOST_DEVICE operator float() const
 #endif
         {
+            assert(T == hip_f8_type::fp8 || T == hip_f8_type::bf8);
             if(T == hip_f8_type::bf8)
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<2, 5, float, true /*negative_zero_nan*/>(data);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    return internal::cast_from_f8<float, false /*is_funz*/>(data, 2, 5, false);
                 else
+#endif
                 {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<2, 5, float, false /*negative_zero_nan*/>(data);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<2, 5, float, true /*negative_zero_nan*/>(data);
+                    }
+                    else
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<2, 5, float, false /*negative_zero_nan*/>(data);
+                    }
                 }
             }
             else /* fp8*/
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<3, 4, float, true /*negative_zero_nan*/>(data);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    return internal::cast_from_f8<float, false /*is_funz*/>(data, 3, 4, false);
                 else
+#endif
                 {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<3, 4, float, false /*negative_zero_nan*/>(data);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<3, 4, float, true /*negative_zero_nan*/>(data);
+                    }
+                    else
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<3, 4, float, false /*negative_zero_nan*/>(data);
+                    }
                 }
             }
         }
@@ -401,35 +579,57 @@ namespace Tensile
             return _Float16(float(*this)); // convert to float, then convert to f16
         }
 
+        explicit inline HIP_HOST operator _Float16() const
+#elif defined(__gfx1200__)
+        explicit inline HIP_DEVICE operator _Float16() const
+        {
+            return _Float16(float(*this)); // convert to float, then convert to f16
+        }
+
         explicit inline HIP_HOST operator _Float16() const
 #else
         explicit inline HIP_HOST_DEVICE operator _Float16() const
 #endif
         {
+            assert(T == hip_f8_type::fp8 || T == hip_f8_type::bf8);
             if(T == hip_f8_type::bf8)
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<2, 5, _Float16, true /*negative_zero_nan*/>(data);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    return internal::cast_from_f8<float, false /*is_funz*/>(data, 2, 5, false);
                 else
+#endif
                 {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<2, 5, _Float16, false /*negative_zero_nan*/>(data);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<2, 5, _Float16, true /*negative_zero_nan*/>(data);
+                    }
+                    else
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<2, 5, _Float16, false /*negative_zero_nan*/>(data);
+                    }
                 }
             }
             else /* fp8*/
             {
-                if(get_hip_f8_bias_mode())
-                {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<3, 4, _Float16, true /*negative_zero_nan*/>(data);
-                }
+#if defined(HIP_FP8_TYPE_OCP)
+                if(IsOCPSupported())
+                    return internal::cast_from_f8<float, false /*is_funz*/>(data, 2, 5, false);
                 else
+#endif
                 {
-                    return tensile_hip_f8_impl::
-                        cast_from_f8<3, 4, _Float16, false /*negative_zero_nan*/>(data);
+                    if(get_hip_f8_bias_mode())
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<3, 4, _Float16, true /*negative_zero_nan*/>(data);
+                    }
+                    else
+                    {
+                        return tensile_hip_f8_impl::
+                            cast_from_f8<3, 4, _Float16, false /*negative_zero_nan*/>(data);
+                    }
                 }
             }
         }
@@ -441,7 +641,11 @@ namespace Tensile
         // check for zero
         inline HIP_HOST_DEVICE bool is_zero() const
         {
-            if(get_hip_f8_bias_mode())
+            if(get_hip_f8_bias_mode()
+#if defined(HIP_FP8_TYPE_OCP)
+               || IsOCPSupported()
+#endif
+            )
             {
                 return data == 0x00;
             }
@@ -454,23 +658,35 @@ namespace Tensile
         // check for nan
         inline HIP_HOST_DEVICE bool is_nan() const
         {
-            if(get_hip_f8_bias_mode())
+#if defined(HIP_FP8_TYPE_OCP)
+            if(IsOCPSupported())
             {
-                return data == 0x80;
+                return (T == hip_f8_type::fp8)   ? ((data & 0x7f) == 0x7f)
+                       : (T == hip_f8_type::bf8) ? ((data & 0x7f) > 0x7c)
+                                                 : false;
             }
             else
+#endif
             {
-                if(T == hip_f8_type::bf8)
+                if(get_hip_f8_bias_mode())
                 {
-                    return (data == 0x7d) || (data == 0x7e) || (data == 0x7f) || (data == 0xfd)
-                           || (data == 0xfe) || (data == 0xff);
+                    return data == 0x80;
                 }
                 else
                 {
-                    return (data == 0x79) || (data == 0x7a) || (data == 0x7b) || (data == 0x7c)
-                           || (data == 0x7d) || (data == 0x7e) || (data == 0x7f) || (data == 0xf9)
-                           || (data == 0xfa) || (data == 0xfb) || (data == 0xfc) || (data == 0xfd)
-                           || (data == 0xfe) || (data == 0xff);
+                    if(T == hip_f8_type::bf8)
+                    {
+                        return (data == 0x7d) || (data == 0x7e) || (data == 0x7f) || (data == 0xfd)
+                               || (data == 0xfe) || (data == 0xff);
+                    }
+                    else
+                    {
+                        return (data == 0x79) || (data == 0x7a) || (data == 0x7b) || (data == 0x7c)
+                               || (data == 0x7d) || (data == 0x7e) || (data == 0x7f)
+                               || (data == 0xf9) || (data == 0xfa) || (data == 0xfb)
+                               || (data == 0xfc) || (data == 0xfd) || (data == 0xfe)
+                               || (data == 0xff);
+                    }
                 }
             }
         }
@@ -478,19 +694,28 @@ namespace Tensile
         // check for inf
         inline HIP_HOST_DEVICE bool is_inf() const
         {
-            if(get_hip_f8_bias_mode())
+#if defined(HIP_FP8_TYPE_OCP)
+            if(IsOCPSupported())
             {
-                return data == 0x80;
+                return (T == hip_f8_type::bf8) ? (data & 0x7f) == 0x7c : false;
             }
             else
+#endif
             {
-                if(T == hip_f8_type::bf8)
+                if(get_hip_f8_bias_mode())
                 {
-                    return (data == 0x7c) || (data == 0xfc);
+                    return data == 0x80;
                 }
                 else
                 {
-                    return (data == 0x78) || (data == 0xf8);
+                    if(T == hip_f8_type::bf8)
+                    {
+                        return (data == 0x7c) || (data == 0xfc);
+                    }
+                    else
+                    {
+                        return (data == 0x78) || (data == 0xf8);
+                    }
                 }
             }
         }
