@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -36,10 +36,11 @@ from . import Utils
 from .TensileInstructions import getGfxName, TensileInstructions
 from .Common import globalParameters, HR, print1, print2, printExit, ensurePath, \
                     CHeader, CMakeHeader, assignGlobalParameters, \
-                    architectureMap
+                    architectureMap, supportedCompiler, printWarning
 from .KernelWriterAssembly import KernelWriterAssembly
 from .SolutionLibrary import MasterSolutionLibrary
 from .SolutionStructs import Solution
+from .CustomYamlLoader import load_logic_gfx_arch
 
 import argparse
 import collections
@@ -52,7 +53,8 @@ import shutil
 import subprocess
 import sys
 from timeit import default_timer as timer
-from copy import deepcopy
+from pathlib import Path
+from typing import Sequence, List
 
 def timing(func):
   def wrapper(*args, **kwargs):
@@ -178,13 +180,13 @@ def getAssemblyCodeObjectFiles(kernels, kernelWriterAssembly, outputPath):
     return coFiles
 
 def which(p):
+    if supportedCompiler(p) and 'CMAKE_CXX_COMPILER' in os.environ and os.path.isfile(os.environ['CMAKE_CXX_COMPILER']):
+        return os.environ['CMAKE_CXX_COMPILER']
     if os.name == "nt":
-      exes = [p+x for x in ['.bat', '', '.exe']]  # bat may be front end for file with no extension
+      exes = [p+x for x in ['.exe', '', '.bat']]  # bat may be front end for file with no extension
     else:
       exes = [p+x for x in ['', '.exe', '.bat']]
     system_path = os.environ['PATH'].split(os.pathsep)
-    if p == 'hipcc' and 'CMAKE_CXX_COMPILER' in os.environ and os.path.isfile(os.environ['CMAKE_CXX_COMPILER']):
-        return os.environ['CMAKE_CXX_COMPILER']
     for dirname in system_path+[globalParameters["ROCmBinPath"]]:
         for exe in exes:
             candidate = os.path.join(os.path.expanduser(dirname), exe)
@@ -236,14 +238,21 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
 
     coFilenames = []
 
-    if (CxxCompiler == "hipcc"):
+    if supportedCompiler(CxxCompiler):
       archs, cmdlineArchs = splitArchs()
 
       archFlags = ['--offload-arch=' + arch for arch in cmdlineArchs]
 
-      hipFlags = ["--genco", "-D__HIP_HCC_COMPAT_MODE__=1"] #needs to be fixed when Maneesh's change is made available
+      # needs to be fixed when Maneesh's change is made available
+      hipFlags = ["-D__HIP_HCC_COMPAT_MODE__=1"]
+      hipFlags += (
+          ["--genco"] if CxxCompiler == "hipcc" else ["--cuda-device-only", "-x", "hip", "-O3"]
+      )
+      # if CxxCompiler == "amdclang++":
+      # hipFlags += ["-mllvm", "-amdgpu-early-inline-all=true", "-mllvm", "-amdgpu-function-calls=false"]
 
       hipFlags += ['-I', outputPath]
+      hipFlags += ["-Xoffload-linker", "--build-id=%s"%globalParameters["BuildIdKind"]]
       hipFlags += ['-std=c++17']
       if globalParameters["SaveTemps"]:
         hipFlags += ['--save-temps']
@@ -252,14 +261,15 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
 
       if os.name == "nt":
         hipFlags += ['-fms-extensions', '-fms-compatibility', '-fPIC', '-Wno-deprecated-declarations']
-        compileArgs = launcher + [which('hipcc')] + hipFlags + archFlags + [kernelFile, '-c', '-o', os.path.join(buildPath, objectFilename)]
+        compileArgs = launcher + [which(CxxCompiler)] + hipFlags + archFlags + [kernelFile, '-c', '-o', os.path.join(buildPath, objectFilename)]
       else:
-        compileArgs = launcher + [which('hipcc')] + hipFlags + archFlags + [kernelFile, '-c', '-o', os.path.join(buildPath, objectFilename)]
+        compileArgs = launcher + [which(CxxCompiler)] + hipFlags + archFlags + [kernelFile, '-c', '-o', os.path.join(buildPath, objectFilename)]
 
       if globalParameters["PrintCodeCommands"]:
-        print('hipcc:', ' '.join(compileArgs))
+        print(CxxCompiler + ':' + ' '.join(compileArgs))
       subprocess.check_call(compileArgs)
 
+      # If we aren't using hipcc what happens?
       # get hipcc version due to compatiblity reasons
       hipccver = globalParameters['HipClangVersion'].split(".")
       hipccMaj = int(hipccver[0])
@@ -339,7 +349,7 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
 
 def buildSourceCodeObjectFiles(CxxCompiler, kernelFiles, outputPath):
     args    = zip(itertools.repeat(CxxCompiler), itertools.repeat(outputPath), kernelFiles)
-    coFiles = Common.ParallelMap(buildSourceCodeObjectFile, args, "Compiling source kernels", method=lambda x: x.starmap)
+    coFiles = Common.ParallelMap2(buildSourceCodeObjectFile, args, "Compiling source kernels")
 
     return itertools.chain.from_iterable(coFiles)
 
@@ -561,13 +571,13 @@ def writeSolutionsAndKernels(outputPath, CxxCompiler, problemTypes, solutions, k
 
   kernelFiles += buildKernelSourceAndHeaderFiles(results, outputPath, kernelsWithBuildErrs)
 
-  kernelsToBuild = list(kernels)
+  kernelsToBuild = kernels
   if errorTolerant:
       def success(kernel):
           writer = kernelWriterAssembly
           kernelName = writer.getKernelName(kernel)
           return kernelName not in kernelsWithBuildErrs
-      kernelsToBuild = list(filter(success, kernelsToBuild))
+      kernelsToBuild = filter(success, kernelsToBuild)
   elif len(kernelsWithBuildErrs) > 0:
     print("\nKernel compilation failed in one or more subprocesses. May want to set CpuThreads=0 and re-run to make debug easier")
     printExit("** kernel compilation failure **")
@@ -821,7 +831,7 @@ def buildObjectFileNames(kernelWriterAssembly, kernels, kernelHelperObjs):
   sourceLibFiles = []
   asmLibFiles = []
 
-  asmKernels = list([k for k in kernels if k['KernelLanguage'] == 'Assembly'])
+  asmKernels = (k for k in kernels if k['KernelLanguage'] == 'Assembly')
 
   # Build a list of kernel object names.
   for kernel in asmKernels:
@@ -829,13 +839,13 @@ def buildObjectFileNames(kernelWriterAssembly, kernels, kernelHelperObjs):
 
   kernelHelperObjNames = [ko.getKernelName() for ko in kernelHelperObjs]
 
-  cxxCompiler = globalParameters["CxxCompiler"]
+  CxxCompiler = globalParameters["CxxCompiler"]
 
   # Source based kernels are built for all supported architectures
-  if (cxxCompiler == 'hipcc'):
+  if supportedCompiler(CxxCompiler):
     sourceArchs, _ = splitArchs()
   else:
-    raise RuntimeError("Unknown compiler %s" % cxxCompiler)
+    raise RuntimeError("Unknown compiler %s" % CxxCompiler)
 
   # Asm based kernels target the configured ISA
   asmArchs = collections.defaultdict(list)
@@ -873,26 +883,26 @@ def buildObjectFileNames(kernelWriterAssembly, kernels, kernelHelperObjs):
     allSources = sourceKernelNames + kernelHelperObjNames
 
     for kernelName in (allSources):
-      if (cxxCompiler == 'hipcc'):
+      if supportedCompiler(CxxCompiler):
         sourceLibFiles += ["%s.so-000-%s.hsaco" % (kernelName, arch) for arch in sourceArchs]
       else:
-        raise RuntimeError("Unknown compiler {}".format(cxxCompiler))
+        raise RuntimeError("Unknown compiler {}".format(CxxCompiler))
   elif globalParameters["NumMergedFiles"] > 1:
-    if (cxxCompiler == 'hipcc'):
+    if supportedCompiler(CxxCompiler):
       for kernelIndex in range(0, globalParameters["NumMergedFiles"]):
         sourceLibFiles += ["Kernels%d.so-000-%s.hsaco" % (kernelIndex, arch) for arch in sourceArchs]
     else:
-      raise RuntimeError("Unknown compiler {}".format(cxxCompiler))
+      raise RuntimeError("Unknown compiler {}".format(CxxCompiler))
   elif globalParameters["LazyLibraryLoading"]:
     fallbackLibs = list(set([kernel._state["codeObjectFile"] for kernel in kernels if "fallback" in kernel._state.get('codeObjectFile', "")]))
     sourceLibFiles += ["{0}_{1}.hsaco".format(name, arch) for name, arch in itertools.product(fallbackLibs, sourceArchs)]
-    if (cxxCompiler == 'hipcc'):
+    if supportedCompiler(CxxCompiler):
       sourceLibFiles += ["Kernels.so-000-%s.hsaco" % (arch) for arch in sourceArchs]
   else: # Merge
-    if (cxxCompiler == 'hipcc'):
+    if supportedCompiler(CxxCompiler):
       sourceLibFiles += ["Kernels.so-000-%s.hsaco" % (arch) for arch in sourceArchs]
     else:
-      raise RuntimeError("Unknown compiler {}".format(cxxCompiler))
+      raise RuntimeError("Unknown compiler {}".format(CxxCompiler))
 
   # Returns names for all xnack versions
   def addxnack(name, ext):
@@ -1070,47 +1080,56 @@ def generateLogicDataAndSolutions(logicFiles, args):
   else:
     archs = args.Architecture.split("_") # workaround for cmake list in list issue
 
-  fIter = zip(logicFiles, itertools.repeat(archs))
-  libraries = Common.ParallelMap(LibraryIO.parseLibraryLogicFile, fIter, "Reading logic files", method=lambda x: x.starmap)
-
   solutions = []
   masterLibraries = {}
   fullMasterLibrary = None
-
   nextSolIndex = 0
+  matchTable = {}
+  fIter = zip(logicFiles, itertools.repeat(archs))
 
-  for logic in filter(lambda i: i[1] != "", Utils.tqdm(libraries, "Processing logic data")):
-    (_, architectureName, _, _, _, newLibrary, _) = logic
+  def libraryIter(lib: MasterSolutionLibrary):
+    if len(lib.solutions):
+      for i, s in enumerate(lib.solutions.items()):
+        yield (i, *s)
+    else:
+      for _, lazyLib in lib.lazyLibraries.items():
+        yield from libraryIter(lazyLib)
+
+  for library in Common.ParallelMap2(LibraryIO.parseLibraryLogicFile, fIter, "Loading Logics...", return_as="generator_unordered"):
+    _, architectureName, _, _, _, newLibrary, srcFile = library
+
+    if architectureName == "":
+      continue
 
     if globalParameters["PackageLibrary"]:
       if architectureName in masterLibraries:
         masterLibraries[architectureName].merge(newLibrary)
       else:
-        masterLibraries[architectureName] = deepcopy(newLibrary)
+        masterLibraries[architectureName] = newLibrary
         masterLibraries[architectureName].version = args.version
     elif globalParameters["SeparateArchitectures"] or globalParameters["LazyLibraryLoading"]:
       if architectureName in masterLibraries:
         nextSolIndex = masterLibraries[architectureName].merge(newLibrary, nextSolIndex)
       else:
-        masterLibraries[architectureName] = deepcopy(newLibrary)
+        masterLibraries[architectureName] = newLibrary
         masterLibraries[architectureName].version = args.version
     else:
       if fullMasterLibrary is None:
-        fullMasterLibrary = deepcopy(newLibrary)
+        fullMasterLibrary = newLibrary
         fullMasterLibrary.version = args.version
       else:
         fullMasterLibrary.merge(newLibrary)
 
-    # if problemType not in logicData:
-    #   logicData[problemType] = []
-    # logicData[problemType].append((scheduleName, deviceNames, \
-    #     solutionsForSchedule, indexOrder, exactLogic, rangeLogic ))
+    if args.GenSolTable:
+      # Match yaml file solutions to solution index
+      for localIdx, _, s in libraryIter(newLibrary):
+        matchTable[s.index] = [srcFile, localIdx]
 
   if globalParameters["SeparateArchitectures"] or globalParameters["LazyLibraryLoading"]:
     if "fallback" in masterLibraries.keys():
       for key, value in masterLibraries.items():
         if key != "fallback":
-          value.merge(deepcopy(masterLibraries["fallback"]))
+          value.merge(masterLibraries["fallback"])
 
       masterLibraries.pop("fallback")
 
@@ -1127,18 +1146,7 @@ def generateLogicDataAndSolutions(logicFiles, args):
   # remove duplicates while preserving order
   solutions = dict.fromkeys(solutions).keys()
 
-  # Match yaml file solutions to solution index
   if args.GenSolTable:
-    matchTable = {}
-    counter = 0
-    for logic in filter(lambda i: i[1] != "", Utils.tqdm(libraries, "Match yaml file solutions to solution index")):
-      (_, architectureName, _, _, _, newLibrary, srcFile) = logic
-      fileCounter = 0
-      for _, s in newLibrary.solutions.items():
-        assert counter == s.index
-        matchTable[s.index] = [srcFile, fileCounter]
-        fileCounter += 1
-        counter += 1
     LibraryIO.write("MatchTable", matchTable)
 
   return solutions, masterLibraries, fullMasterLibrary
@@ -1174,15 +1182,18 @@ def WriteClientLibraryFromSolutions(solutionList, libraryWorkingPath, tensileSou
 
   if tensileSourcePath == None:
     tensileSourcePath = os.path.dirname(os.path.realpath(__file__))
-  firstSolution = deepcopy(solutionList[0])
+  firstSolution = solutionList[0]
   problemType = firstSolution["ProblemType"].state
   problemType["DataType"] = problemType["DataType"].value
   problemType["DataTypeA"] = problemType["DataTypeA"].value
   problemType["DataTypeB"] = problemType["DataTypeB"].value
   problemType["DataTypeE"] = problemType["DataTypeE"].value
+  problemType["DataTypeAmaxD"] = problemType["DataTypeAmaxD"].value
   problemType["DestDataType"] = problemType["DestDataType"].value
   problemType["ComputeDataType"] = problemType["ComputeDataType"].value
   problemType["F32XdlMathOp"] = problemType["F32XdlMathOp"].value
+  if "DataTypeMetadata" in problemType:
+    problemType["DataTypeMetadata"] = problemType["DataTypeMetadata"].value
   cxxCompiler = globalParameters["CxxCompiler"]
 
   effectiveWorkingPath = os.path.join(libraryWorkingPath, "library")
@@ -1196,6 +1207,23 @@ def WriteClientLibraryFromSolutions(solutionList, libraryWorkingPath, tensileSou
 
   return (codeObjectFiles, newLibrary)
 
+def validateLibrary(masterLibraries: MasterSolutionLibrary,
+                    kernels: Sequence[Solution],
+                    kernelWriterAssembly: KernelWriterAssembly):
+  kernelsByCodeObjectFiles = {k: list(g) for k, g in itertools.groupby(kernels, lambda k: k["codeObjectFile"])}
+
+  ok: bool = True
+
+  for _, lib in masterLibraries.items():
+    for name, llib in lib.lazyLibraries.items():
+      uniqueKernelsInLib = {kernelWriterAssembly.getKernelName(s.originalSolution) for s in llib.solutions.values()}
+
+      if len(uniqueKernelsInLib) != len(kernelsByCodeObjectFiles[name]):
+        ok = False
+        print(f"{name} library and co has inconsistent kernel size {len(uniqueKernelsInLib)} vs {len(kernelsByCodeObjectFiles[name])}")
+
+  assert ok and "Inconsistent kernel sizes detected!"
+
 ################################################################################
 # Tensile Create Library
 ################################################################################
@@ -1224,7 +1252,7 @@ def TensileCreateLibrary():
   argParser.add_argument("LogicPath",       help="Path to LibraryLogic.yaml files.")
   argParser.add_argument("OutputPath",      help="Where to write library files?")
   argParser.add_argument("RuntimeLanguage", help="Which runtime language?", choices=["OCL", "HIP", "HSA"])
-  argParser.add_argument("--cxx-compiler",           dest="CxxCompiler",       choices=["hipcc"],       action="store", default="hipcc")
+  argParser.add_argument("--cxx-compiler",           dest="CxxCompiler",       choices=["hipcc", "amdclang++"], action="store", default="amdclang++")
   argParser.add_argument("--cmake-cxx-compiler",     dest="CmakeCxxCompiler",  action="store")
   argParser.add_argument("--code-object-version",    dest="CodeObjectVersion", choices=["default", "V4", "V5"], action="store")
   argParser.add_argument("--architecture",           dest="Architecture",      type=str, action="store", default="all", help="Supported archs: " + " ".join(architectureMap.keys()))
@@ -1245,6 +1273,8 @@ def TensileCreateLibrary():
   argParser.add_argument("--version", help="Version string to embed into library file.")
   argParser.add_argument("--generate-manifest-and-exit",   dest="GenerateManifestAndExit", action="store_true",
                           default=False, help="Output manifest file with list of expected library objects and exit.")
+  argParser.add_argument("--logic-format", dest="LogicFormat", choices=["yaml", "json"], \
+                         action="store", default="yaml", help="select which logic format to use")
   argParser.add_argument("--library-format", dest="LibraryFormat", choices=["yaml", "msgpack"],
                          action="store", default="msgpack", help="select which library format to use")
   argParser.add_argument("--generate-sources-and-exit",   dest="GenerateSourcesAndExit", action="store_true",
@@ -1264,10 +1294,14 @@ def TensileCreateLibrary():
   argParser.add_argument("--client-config", dest="ClientConfig", action="store_true",
                          help="Create client config for setting the library and code object files")
   argParser.add_argument("--global-parameters", nargs="+", type=splitExtraParameters, default=[])
-  argParser.add_argument("--generate-solution-table", dest="GenSolTable", action="store_true", default=True,
-                         help="Generate solution-yaml matching table")
+  argParser.add_argument("--no-generate-solution-table", dest="GenSolTable", action="store_false", default=True,
+                         help="Skip generating solution-yaml matching table")
   argParser.add_argument("--asm-debug", dest="AsmDebug", action="store_true", default=False,
                          help="Keep debug information for built code objects")
+  argParser.add_argument("--build-id", dest="BuildIdKind", action="store", default="sha1")
+  argParser.add_argument("--keep-build-tmp", dest="KeepBuildTmp", action="store_true",
+                          default=False, help="Do not remove the temporary build directory (may required hundreds of GBs of space)"),
+  argParser.add_argument("--validate-library", dest="ValidateLibrary", action="store_true", default=False)
 
   args = argParser.parse_args()
 
@@ -1293,6 +1327,7 @@ def TensileCreateLibrary():
   arguments["LibraryPrintDebug"] = args.LibraryPrintDebug
   arguments["CodeFromFiles"] = False
   arguments["EmbedLibrary"] = args.EmbedLibrary
+  arguments["LogicFormat"]  = args.LogicFormat
   arguments["LibraryFormat"] = args.LibraryFormat
   if args.no_enumerate:
     arguments["ROCmAgentEnumeratorPath"] = False
@@ -1309,6 +1344,9 @@ def TensileCreateLibrary():
   arguments["PrintLevel"] = args.PrintLevel
   arguments["PrintTiming"] = args.PrintTiming
   arguments["AsmDebug"] = args.AsmDebug
+  arguments["BuildIdKind"] = args.BuildIdKind
+  arguments["KeepBuildTmp"] = args.KeepBuildTmp
+  arguments["ValidateLibrary"] = args.ValidateLibrary
 
   for key, value in args.global_parameters:
     arguments[key] = value
@@ -1338,12 +1376,25 @@ def TensileCreateLibrary():
     printExit("--lazy-library-loading requires --merge-files and --separate-architectures enabled")
 
   # Recursive directory search
+  logicExtFormat = ".yaml"
+  if args.LogicFormat == "yaml":
+    pass
+  elif args.LogicFormat == "json":
+    logicExtFormat = ".json"
+  else:
+    printExit("Unrecognized LogicFormat", args.LogicFormat)
+
+  def archMatch(arch: str, archs: List[str]):
+    return (arch in archs) or any(a.startswith(arch) for a in archs)
+
+  def validLogicFile(p: Path):
+    return p.suffix == logicExtFormat and ("all" in archs or archMatch(load_logic_gfx_arch(p), archs))
+
   logicFiles = []
-  for root, dirs, files in os.walk(logicPath):
-    logicFiles += [os.path.join(root, f) for f in files
-                       if os.path.splitext(f)[1]==".yaml" \
-                       and (any(logicArch in os.path.splitext(f)[0] for logicArch in logicArchs) \
-                       or "hip" in os.path.splitext(f)[0]) ]
+
+  for root, _, files in os.walk(logicPath):
+    logics = (os.path.join(root, f) for f in files)
+    logicFiles += [file for file in logics if validLogicFile(Path(file))]
 
   print1("# LibraryLogicFiles:" % logicFiles)
   for logicFile in logicFiles:
@@ -1361,6 +1412,9 @@ def TensileCreateLibrary():
   # if any kernels are assembly, append every ISA supported
   kernelWriterAssembly, kernelMinNaming, _ = getSolutionAndKernelWriters(solutions, kernels)
 
+  if globalParameters["ValidateLibrary"]:
+    validateLibrary(masterLibraries, kernels, kernelWriterAssembly)
+
   staticFiles = copyStaticFiles(outputPath)
 
   # Build a list of files to be expected
@@ -1498,6 +1552,13 @@ def TensileCreateLibrary():
 
   checkFileExistence(itertools.chain(libMetadataPaths, sourceLibPaths, asmLibPaths))
 
+  if not globalParameters["KeepBuildTmp"]:
+    buildTmp = Path(outputPath).parent / "library" / "build_tmp"
+    if buildTmp.exists() and buildTmp.is_dir():
+      shutil.rmtree(buildTmp)
+    else:
+      printWarning(f"Cannot remove {str(buildTmp)}")
+
   print1("# Tensile Library Writer DONE")
   print1(HR)
   print1("")
